#include "../SDK/foobar2000.h"
#include "wavpack/wputils.h"
#include "wavpacklib/wavpack_common.h"
#include "wavpacklib/wavpack_frame.h"
#include "wavpacklib/wavpack_buffer_decoder.h"

DECLARE_COMPONENT_VERSION("WavPack Packet Decoder", "0.9.0", "Base on Official WavePack Library & wavpacklib by Toff\n(c)2006 Haru Ayana <ayana@reharmonize.net>");

typedef struct {
	short version;
} wavpack_codec_private_data;

class packet_decoder_wavpack : public packet_decoder
{
public:
    packet_decoder_wavpack()
    {
		m_WBD = NULL;
		m_MainFrame = NULL;
		m_CorrectionFrame = NULL;
		m_SampleRate = m_Channels = 0;
		m_FrameFlag = 0;
    }

    ~packet_decoder_wavpack()
    {
		cleanup();
    }

	virtual t_size set_stream_property(const GUID & p_type,t_size p_param1,const void * p_param2,t_size p_param2size)
	{
		return 0;
	}

	virtual void get_info(file_info & p_info)
	{
		p_info.info_set_int("samplerate", m_SampleRate);
		p_info.info_set_int("channels", m_Channels);
		p_info.info_set("codec", "WavPack");
		if (m_WBD->wpc != NULL) {
			p_info.info_set_int("bitspersample", WavpackGetBitsPerSample(m_WBD->wpc));
			int mode = WavpackGetMode(m_WBD->wpc);
			if (mode & MODE_HIGH) {
				p_info.info_set("codec_profile", "high");
			} else if (mode & MODE_FAST) {
				p_info.info_set("codec_profile", "fast");
			} else if (mode & MODE_EXTRA) {
				p_info.info_set("codec_profile", "extra");
			} else {
				p_info.info_set("codec_profile", "normal");
			}
			if (mode & MODE_HYBRID) {
				p_info.info_set("codec_profile", "hybrid");
				p_info.info_set("encoding", "hybrid");
			} else if (mode & MODE_LOSSLESS) {
				p_info.info_set("encoding", "lossless");
			} else {
				p_info.info_set("encoding", "lossy");
			}
			unsigned char md5[16] = "";
			if (WavpackGetMD5Sum(m_WBD->wpc, md5)) {
				p_info.info_set("md5", (const char*)md5);
			}
		}
	}

	virtual unsigned get_max_frame_dependency() { return 0; }
	virtual double get_max_frame_dependency_time() { return 0; }

	virtual void reset_after_seek()
	{
	}

	virtual void decode(const void * p_buffer,t_size p_bytes,audio_chunk & p_chunk,abort_callback & p_abort)
	{
		if (m_WBD == NULL || m_MainFrame == NULL) return;
		bool is_several_blocks = (m_Channels > 2);
		reconstruct_wavpack_frame(m_MainFrame, &m_CommonFrameData, (char*)p_buffer, (uint32_t)p_bytes, true, is_several_blocks, m_CodecPrivateData.version);
		if (!wavpack_buffer_decoder_load_frame(m_WBD, m_MainFrame->data, m_MainFrame->len, NULL, 0)) {
			console::error(uStringPrintf("foo_packet_decoder_wavpack: %s", m_WBD->wavpack_error_msg));
			cleanup();
			throw exception_io_unsupported_format();
		}
		int num_samples = m_CommonFrameData.block_samples;
		m_Buffer.set_size(num_samples * m_Channels * 4);
		uint32_t samples = wavpack_buffer_decoder_unpack(m_WBD, m_Buffer.get_ptr(), num_samples);
		if (samples) {
			m_Buffer.set_size(samples * m_Channels * WavpackGetBytesPerSample(m_WBD->wpc));
			wavpack_buffer_format_samples(m_WBD, (uchar*)m_Buffer.get_ptr(), (long*)m_Buffer.get_ptr(), samples);
			p_chunk.set_data_fixedpoint(m_Buffer.get_ptr(), m_Buffer.get_size(), m_SampleRate, m_Channels, WavpackGetBitsPerSample(m_WBD->wpc), audio_chunk::g_guess_channel_config(m_Channels));
		}
	}

	virtual bool analyze_first_frame_supported() { return true; }

	virtual void analyze_first_frame(const void * p_buffer,t_size p_bytes,abort_callback & p_abort)
	{
		if (m_WBD == NULL || m_MainFrame == NULL) return;
		bool is_several_blocks = (m_Channels > 2);
		reconstruct_wavpack_frame(m_MainFrame, &m_CommonFrameData, (char*)p_buffer, (uint32_t)p_bytes, true, is_several_blocks, m_CodecPrivateData.version);
		wavpack_buffer_decoder_load_frame(m_WBD, m_MainFrame->data, m_MainFrame->len, NULL, 0);
		m_FrameFlag = m_CommonFrameData.array_flags[0];
	}

	void open(const GUID & p_owner,bool p_decode,t_size p_param1,const void * p_param2,t_size p_param2size,abort_callback & p_abort)
	{
		cleanup();
		const matroska_setup* setup = (const matroska_setup*)p_param2;
		m_SampleRate = setup->sample_rate;
		m_Channels = setup->channels;
		m_WBD = wavpack_buffer_decoder_new();
		m_MainFrame = frame_buffer_new();
		m_CorrectionFrame = frame_buffer_new();
		memcpy(&m_CodecPrivateData, setup->codec_private, setup->codec_private_size);
	}

	static bool g_is_our_setup(const GUID & p_owner,t_size p_param1,const void * p_param2,t_size p_param2size) {
		if (p_owner == owner_matroska)
		{
			if (p_param2size == sizeof(matroska_setup))
			{
				const matroska_setup * setup = (const matroska_setup*) p_param2;
				if (!strncmp(setup->codec_id,"A_WAVPACK4",10)) {
					return true;
				} else return false;
			}
			else return false;
		}
		else return false;
	}

private:
	void cleanup()
	{
		if (m_WBD != NULL) {
			wavpack_buffer_decoder_free(m_WBD);
			m_WBD = NULL;
		}
		if (m_MainFrame != NULL) {
			frame_buffer_free(m_MainFrame);
			m_MainFrame = NULL;
		}
		if (m_CorrectionFrame != NULL) {
			frame_buffer_free(m_CorrectionFrame);
			m_CorrectionFrame = NULL;
		}
	}

	wavpack_buffer_decoder* m_WBD;
	wavpack_codec_private_data m_CodecPrivateData;
	common_frame_data m_CommonFrameData;
	frame_buffer* m_MainFrame;
	frame_buffer* m_CorrectionFrame;
	unsigned int m_SampleRate;
	unsigned int m_Channels;
	pfc::array_t<t_int32> m_Buffer;
	unsigned int m_BufferSize;
	unsigned int m_FrameFlag;
};

static packet_decoder_factory_t<packet_decoder_wavpack> g_packet_decoder_wavpack_factory;
